%% Main NSGA-II optimal path planner 
% *************************************************************************
% This is the main program for the NSGA-II based optimal path planner
% designed to take on autonomous docking missions for the ROV Minerva 2.
% Followingly, a description of the subfiles and functions and a user-guide
% for definintion of the path planning problem:
%
% 1. parameters.m: Contains the optimal path planning parameters and the
%               NSGA-II parameters. This file should be modified if other
%               path planning scenarios are to be examined.
%
% 2. initialize_population.m: This function creates an initial population
%                           of size 2N at random based on the input from 
%                           parameters.m. This function calls the function
%                           OFE_multiple_obstacles() for objective functon
%                           evaluation of all the members, i.e. paths, in
%                           the population. The function returns a matrix
%                           where each row corrsponds to one path.
%
% 3. NDS_CDA.m: This function handles the non-dominated sorting and
%           crowding distance assigment of the population in question. The
%           function appends rank and crowding distance to the matrix
%           containing information about the population
%
% 4. BTS.m: This function carries out binary tournament selection based on
%           an input population and returns the winners of the tournament.
%
% 5. make_new_pop.m: This function returns an offspring population based
%                   on a parent population utilizing the genetic operators
%                   simulated binary crossover and polynomial mutation. The 
%                   function calls the function OFE_multiple_obstacles() 
%                   for objective functon evaluation of all the members,
%                   i.e. paths, in the population.
%
% 6. selection.m: This function carries out selection of paths for
%               reproduction, i.e. determines the parent population,
%               based on rank and crowding distance. If paths cannot be
%               distinguished from one another, the function
%               calls BTS() to complete the selection such that the size of
%               the parent population becomes N.
%
% 7. path_selector.m: This is a post-processing file selecting one path
%                   among the obtained Pareto-optimal paths that cannot be
%                   distinguished from one another. This file should be
%                   modified in accordance with the preferred properties of
%                   the selected path.
%
% 8. plot_tool.m. This file plots the results of the optimization, namely,
%               the objective functions values of the final population
%               compared among one another and the selected path in the
%               defined environment.
%
% For more background on the theory and method behind the development of
% this path planner, see:
% Hansen, Tobias Lars, "A NSGA-II Based Optimal Path Planner for Autonomous
% Docking of ROV Using Waypoint Guidance", 2019. 
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%% 
clear all; close all; clc;

%% Initializing parameters
parameters;

%% Generating random population R_0
% R_0: size 2*N
% Evaluating objective functions for initial population, i.e. calls OFE()
R_0 = initialize_population(N, V, K, M, start_pos, final_pos,...
                                    min_range, max_range,...
                                    min_dist, obs_vec, z_terrain, pen_pos);
                                
%% Non-dominated sorting and crowding distance assignment of R_0
R_0 = NDS_CDA(R_0, V, K, M);

%% Binary Tournament Selection - generating initial parent population P_0
% P_0: size N
P = BTS(R_0, N, V, K, M);

%% Generating initial offspring population Q_0
% Q_0: size N
% Objective function evaluation is carried out for offspring population,
% i.e. calls OFE()
Q = make_new_pop(P, V, K, M,... 
                 p_c, eta_c, p_m, eta_m,...
                 min_range, max_range,...
                 min_dist, obs_vec, start_pos, final_pos,...
                 z_terrain, pen_pos);

%% Main loop
for i = 1:G
    %% Intermediate Solution R
    % R_1: size 2*N
    % R_1 is obtained by concatenation of P_0 and Q_0
    R(1:N,:) = P;
    R(N+1:2*N,:) = Q;
    
    %% Non-dominated sorting and crowding distance assignment of R
    R = NDS_CDA(R, V, K, M);
    
    %% Selecting next parent generation, P
    P = selection(R, N, V, K, M);
    
    %% Generating next offspring population Q
    Q = make_new_pop(P, V, K, M,... 
                 p_c, eta_c, p_m, eta_m,...
                 min_range, max_range,...
                 min_dist, obs_vec, start_pos, final_pos,...
                 z_terrain, pen_pos);
                 
end

%% Post processing
% Selecting path according to preferences
path_selector;
nbPath = a_index;
% plotting results
plot_tool;