function f = make_new_pop(x, v_dim, k_interior, M,...
                            p_c, eta_c, p_m, eta_m,...
                            min_range, max_range,...
                            min_dist, obs_vec, start_pos, final_pos,...
                            z_terrain, pen_pos)
%% Description
% *************************************************************************
% This function returns generated offsping population, f, generated by
% applying the genetic operators simulated binary crossover and polynomial
% mutation. The input is the parent population.
%
% This function calls the function OFE_multiple_obstacles for evaluation of
% the objective function values of the generated paths.
%
% For additional explanation of inputs, see parameters.m / Main.m
% *************************************************************************
% Programmed by: Tobias Lars Hansen, 2019.
% *************************************************************************
%%
[N, ~] = size(x);
nbFixedPoints = 3; 
V = v_dim*(k_interior + nbFixedPoints);
p_i = randperm(N);
nb = k_interior*nbFixedPoints;
vmax = repmat(max_range, 1, nb);
vmin = repmat(min_range, 1, nb);
child_pop = zeros(N,V+M+2);
for p = 1:N
    u = zeros(1, V);
    % SBX
    if rand(1) < p_c && p < N-1
        parent_1 = x(p_i(p), 1:V);
        parent_2 = x(p_i(p+1), 1:V);
        
        beta = zeros(1, V);
        alpha = zeros(1, V);
        beta_q = zeros(1,V);

        % Variable by variable
        for i = 1:V
           if parent_1(i) < parent_2(i)
               beta(i) = 1 + (2/(parent_2(i)-parent_1(i)))...
                   *min((parent_1(i)-vmin(i)),(vmax(i)-parent_2(i)));
           else % parent_1 > parent_2
               beta(i) = 1 + (2/(parent_1(i)-parent_2(i)))...
                   *min((parent_2(i)-vmin(i)),(vmax(i)-parent_1(i)));
           end

            alpha(i) = 2 - beta(i)^(eta_c  + 1);
            u(i) = rand(1);
            
            if u(i) <= 1/alpha(i)
                beta_q(i) = ( u(i)*alpha(i) )^( 1/(eta_c + 1) );
            else
                beta_q(i) = (1/(2- u(i)*alpha(i)))^( 1/(eta_c + 1) );
            end
        end

        child_1 =0.5*((parent_1 + parent_2)-beta_q.*(parent_2 - parent_1));
        child_2 =0.5*((parent_1 + parent_2)+beta_q.*(parent_2 - parent_1));
        
        child_pop(p, 1:V) = child_1;
        child_pop(p+1, 1:V) = child_2;
        
        % If SBX occurs, p needs to be changed
        p = p + 1;
        
    % Polynomial mutation
    else

        parent_mut = x(p_i(p), 1:V);
        delta = zeros(1, V);
        delta_q = zeros(1, V);
        child_mut = zeros(1,V);
        
        for i = 1:V
            u(i) = rand(1);
            delta(i) = min( (parent_mut(i) - vmin(i)),....
                            (vmax(i) - parent_mut(i)) )...
                            /(vmax(i) - vmin(i));
            if u(i) < 0.5
                delta_q(i) = (2*u(i)+(1-2*u(i))*(1-delta(i))...
                    ^(eta_m + 1))^(1/(eta_m + 1)) - 1;
            else
                delta_q(i) = 1 - (2*(1-u(i)) + ...
                    2*(u(i) - 0.5)*(1 - delta(i))...
                    ^(eta_m + 1))^(1/(eta_m + 1));
            end
            child_mut(i) = parent_mut(i) + delta_q(i)*(vmax(i) - vmin(i));
        end
        child_pop(p, 1:V) = child_mut;
        
        % Manipulating the fixed point, i.e. start and destinaton
        child_pop(p, 1:3) = start_pos;
        child_pop(p, V-5:V-3) = pen_pos;
        child_pop(p, V-2:V) = final_pos;      
    end  
end %for genetic operations
%% Evaluating objective functions for offspring population
for p = 1:N
    child_pop(p, V+1: V + M) = OFE_multiple_obstacles(child_pop(p,:),...
                                            k_interior, v_dim,...
                                            min_dist, obs_vec, z_terrain);
end
f = child_pop;
end %fcn